import os

from flask import Flask, render_template, request, jsonify
from flask_cors import CORS  # Import CORS
import paramiko
import json
import dotenv
import time

# Enable CORS for all routes
app = Flask(__name__)
CORS(app)

# SSH Configuration
SERVER_1 = 'andorra.imp.fu-berlin.de'
SERVER_2 = 'moritz.imp.fu-berlin.de'
REMOTE_PORT = 22
USERNAME = None
PASSWORD = None


# Function to connect to the first server
def ssh_to_server_1(server1_ip, username1, password1):
    client = paramiko.SSHClient()
    client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    client.connect(server1_ip, username=username1, password=password1)
    return client


# Function to tunnel through the first server and connect to the second server
def ssh_to_server_2_through_server_1(server1_client, server2_ip, username2, password2):
    # Create a transport (tunnel) using the connection from server 1
    transport = server1_client.get_transport()

    # Open a direct TCP connection from server1 to server2
    dest_addr = (server2_ip, 22)  # destination address (server 2)
    local_addr = ('127.0.0.1', 0)  # bind address for local end of the tunnel
    channel = transport.open_channel("direct-tcpip", dest_addr, local_addr)

    # Now establish SSH connection to server2 using the tunnel through server1
    server2_client = paramiko.SSHClient()
    server2_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())

    # Connect to server 2 using the channel (through server 1)
    server2_client.connect(server2_ip, username=username2, password=password2, sock=channel)
    return server2_client


def execute_remote_command(command):
    """Connects to the remote server over SSH and executes a command."""
    try:
        # Connect to the first server
        server1_client = ssh_to_server_1(SERVER_1, USERNAME, PASSWORD)
        print("Connected to server 1")

        # Connect to the second server through server 1
        server2_client = ssh_to_server_2_through_server_1(server1_client, SERVER_2, USERNAME, PASSWORD)
        print("Connected to server 2 through server 1")

        # Execute the command on the remote server
        stdin, stdout, stderr = server2_client.exec_command(command)

        # Retrieve the output and error
        output = stdout.read().decode()
        error = stderr.read().decode()

        server2_client.close()
        server1_client.close()
        print("returning output\n")
        return {"output": output}
    except Exception as e:
        return {"error": str(e)}


@app.route('/')
def index():
    return render_template('index.html')


@app.route('/submit', methods=['POST'])
def submit():
#    time.sleep(1)
#    return jsonify(""".center-checkbox {
#    display: flex;
#    justify-content: center;
#    align-items: center;
#}""")
    data = request.json
    prompt = data['prompt']

    return get_llm_response(prompt)

@app.route('/explain', methods=['POST'])
def explain():
#    time.sleep(1)
#    return jsonify(""".center-checkbox {
#    display: flex; // uses display flex style
#    justify-content: center; // align the items in the box horizontally centered
#    align-items: center; // align the items in the box vertically centered
#}""")
    data = request.json
    code = data['code']
    system_prompt = "You are a code snippet generator. You will get a code snippet. Take that code snippet, do not change it. Only add comments in the code to explain it to a beginner. No explanations or messages outside of the code. No quotes around the code. No language specification."
    return get_llm_response(code, system_prompt)


def get_llm_response(prompt, system_prompt=None, seed=42, temperature=1.0, top_k=40):
    if system_prompt is None:
        system_prompt = "You are a code snippet generator. Only generate code. Use best practices. As short as possible. No options. No explanation. No comment. No quotes around the code. No language specification."

    payload = {
        "model": "llama3.1",
        # "prompt": system_user_prompt + user_prompt,
        "stream": False,
        "options": {
            "seed": seed,
            "temperature": temperature,
            "top_k": top_k
        },
        "messages": [
            {
                "role": "system",
                "content": system_prompt
            },
            {
                "role": "user",
                "content": prompt
            }
        ],
    }
    # Convert the payload to a JSON string for sending via SSH
    payload_json = json.dumps(payload)
    # Prepare the curl command to run on the remote server
    curl_command = f"curl http://localhost:11434/api/chat -d '{payload_json}'"
    # Execute the curl command on the remote server via SSH
    result = execute_remote_command(curl_command)
    result = json.loads(result['output'])
    print(result)
    return jsonify(result['message']['content'])


if __name__ == '__main__':
    dotenv.load_dotenv()
    if not PASSWORD or not USERNAME:
        USERNAME = os.getenv('USERNAME')
        PASSWORD = os.getenv('PW')
    app.run()
